library(tidyverse)
library(lubridate)
library(dplyr)
library(ggplot2)
library(leaflet)
library(shiny)
library(shinydashboard)

df <- read.csv(file = "crime.csv", header = TRUE, sep=",")

#Preprocessing
df$YEAR <- as.factor(df$YEAR)
df$HOUR <- as.factor(df$HOUR)
df$MONTH <- as.factor(df$MONTH)
df$OCCURRED_ON_DATE <- as.POSIXct(df$OCCURRED_ON_DATE, format = "%Y-%m-%d %H:%M:%OS")



  

row1 <- fluidRow( 
  tabBox(
    title = "Crime Frequency",
    id = "tabset1", height = "250px",
    tabPanel("Per Month", plotOutput("permonth")),
    tabPanel("Per Weekday", plotOutput("perday")),
    tabPanel("Per Hour", plotOutput("perhour"))
  )
  ,box(title = "Map View", solidHeader = TRUE 
       ,collapsible = TRUE, leafletOutput("mymap")) 
)

#Creating another fluid row
row2 <- fluidRow( 
  box(title = "Stacked Plot", solidHeader = TRUE 
      ,collapsible = TRUE, plotOutput("stacked"))
  
  ,box(title = "Timeseries Plot", solidHeader = TRUE 
       ,collapsible = TRUE, plotOutput("timeseries")) 
)


#Adding components to body:
body <- dashboardBody(row1, row2)



## Project Boston Crimes: UI - Header & Sidebars
header <- dashboardHeader(title = "Boston Crime Dataset")

sidebar <- dashboardSidebar(
  radioButtons("year", "Select Year: ",
               c("2015" = "2015",
                 "2016" = "2016",
                 "2017" = "2017",
                 "2018" = "2018",
                 "2019" = "2019")),
  
  selectInput("offense",
              "Offense Description:",
              c(unique(as.character(df$OFFENSE_DESCRIPTION[1:10]))))
  
)

#Adding all components to UI:
ui <- dashboardPage(header, sidebar, body)



## Project Boston Crimes: Server
server <- function(input, output) { 

  #creating the plotOutput content
  output$permonth <- renderPlot({
    df1 <- filter(df, YEAR == input$year)
    ggplot(data = df1,aes(x=df1$MONTH)) + geom_bar(fill="steelblue") + xlab("Month") + ylab("Number of Crimes") + theme_minimal()
  })

  output$perday <- renderPlot({
    df1 <- filter(df, YEAR == input$year)
    ggplot(data = df1,aes(x=df1$DAY_OF_WEEK)) + geom_bar(fill="red") + xlab("Day of Week") + ylab("Number of Crimes") + theme_minimal()
  })

  output$perhour <- renderPlot({
    df1 <- filter(df, YEAR == input$year)
    ggplot(data = df1,aes(x=df1$HOUR)) + geom_bar(fill="purple") + xlab("Hour") + ylab("Number of Crimes") + theme_minimal()
  })

  #Defining the timeseries plot
  output$timeseries <- renderPlot({
    
    df_fil_off <- filter(df, OFFENSE_DESCRIPTION == input$offense)
    ggplot(data = df_fil_off, aes(x = df_fil_off$OCCURRED_ON_DATE))+ xlab("Date") + ylab("Number of Crimes") + geom_freqpoly(colour="green")
  })

  #Defining the newly created stacked plot
  output$stacked <- renderPlot({
    df %>% filter(STREET %in% (df  %>% count(STREET) %>% arrange(-n) %>% head(10) %>% pull(STREET)),
                  OFFENSE_DESCRIPTION %in% (df  %>% count(OFFENSE_DESCRIPTION) %>% arrange(-n) %>% head(5) %>% pull(OFFENSE_DESCRIPTION))) %>% 
      ggplot(aes(STREET, fill = OFFENSE_DESCRIPTION))+
      geom_bar(position = "fill")+
      scale_fill_ordinal()+
      coord_flip()
  })

  output$mymap <- renderLeaflet({

    df_fil_off <- filter(df, OFFENSE_DESCRIPTION == input$offense)
    dm <- data.frame(latitude = df_fil_off$Lat[1:500], longitude = df_fil_off$Long[1:500])
    
    leaflet(dm) %>% addProviderTiles("Stamen.TonerLite") %>%  
      addMarkers(
        clusterOptions = markerClusterOptions()
      ) %>% setView(-71.1, 42.32, zoom = 12)
    
  })
  
}



## Project Boston Crimes: Building Our App
shinyApp(ui, server)
